@@ -4,8 +4,6 @@ Connect to a `Slack <https://slack.com>`_ workspace as a bot.
 Config:
     token (str):
         Slack API token for a bot user.
-    fallback-name (str):
-        Name to display for incoming messages without an attached user (default: ``IMMP``).
     fallback-image (str):
         Avatar to display for incoming messages without a user or image (default: none).
     thread-broadcast (bool):
@@ -30,6 +28,7 @@ from functools import partial
 from json import dumps as json_dumps
 import logging
 import re
+import time
 
 from aiohttp import ClientResponseError, FormData
 from emoji import emojize
@@ -48,7 +47,6 @@ class _Schema:
                for size in image_sizes}
 
     config = immp.Schema({"token": str,
-                          immp.Optional("fallback-name", "IMMP"): str,
                           immp.Optional("fallback-image"): immp.Nullable(str),
                           immp.Optional("thread-broadcast", False): bool})
 
@@ -100,7 +98,6 @@ class _Schema:
                  immp.Optional("edited", dict):
                      {immp.Optional("user"): immp.Nullable(str)},
                  immp.Optional("thread_ts"): immp.Nullable(str),
-                 immp.Optional("replies", list): [{"ts": str}],
                  immp.Optional("files", list): [file],
                  immp.Optional("attachments", list): [attachment],
                  immp.Optional("is_ephemeral", False): bool}
@@ -562,33 +559,22 @@ class SlackMessage(immp.Message):
             action = True
             # Slack leaves a leading space in the message text: " set up a reminder..."
             text = text.lstrip()
-        thread = recent = None
-        if event["thread_ts"]:
+        if event["thread_ts"] and event["ts"] != event["thread_ts"] and parent:
             thread = immp.Receipt(event["thread_ts"], immp.Channel(slack, event["channel"]))
-        if thread and parent:
-            try:
-                thread = await slack.get_message(thread, False)
-            except MessageNotFound:
-                pass
-        if isinstance(thread, immp.Message):
-            for reply in thread.raw["replies"][::-1]:
-                if reply["ts"] not in (event["ts"], event["thread_ts"]):
-                    # Reply to a thread with at least one other message, use the next most
-                    # recent rather than the parent.
-                    recent = immp.Receipt(reply["ts"], channel)
+            # Look for the current reply in the thread, and take the previous message as reply-to.
+            last = None
+            for entry in await slack.get_replies(thread):
+                if entry.id == event["ts"]:
                     break
-        if recent and parent:
+                last = entry
             try:
-                recent = await slack.get_reply(channel.source, event["thread_ts"],
-                                               recent.id, False)
+                if last:
+                    reply_to = await slack.get_reply(thread, last.id, False)
+                else:
+                    # No previous reply, take the thread parent message as reply-to instead.
+                    reply_to = await slack.get_message(thread, False)
             except MessageNotFound:
-                pass
-        if thread and recent:
-            # Don't walk the whole thread, just link to the parent after one step.
-            recent.reply_to = thread
-            reply_to = recent
-        else:
-            reply_to = recent or thread
+                reply_to = last or thread
         for file in event["files"]:
             try:
                 attachments.append(SlackFile.from_file(slack, file))
@@ -955,9 +941,22 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
         log.debug("Failed to find message %r in %r", receipt.id, receipt.channel.source)
         raise MessageNotFound
 
-    async def get_reply(self, channel_id, thread_ts, reply_ts, parent=True):
-        params = {"channel": channel_id,
-                  "ts": thread_ts,
+    async def get_replies(self, receipt):
+        params = {"channel": receipt.channel.source, "ts": receipt.id}
+        try:
+            replies = await self._api("conversations.replies", _Schema.history, params=params)
+        except SlackAPIError as e:
+            log.debug("API error retrieving replies %r from %r: %r",
+                      receipt.id, receipt.channel.source, e.args[0])
+            raise MessageNotFound from None
+        else:
+            return [immp.Receipt(msg["ts"], receipt.channel,
+                                 at=datetime.fromtimestamp(float(msg["ts"]), timezone.utc))
+                    for msg in replies["messages"] if msg["ts"] != receipt.id]
+
+    async def get_reply(self, receipt, reply_ts, parent=True):
+        params = {"channel": receipt.channel.source,
+                  "ts": receipt.id,
                   "latest": reply_ts,
                   "inclusive": "true",
                   "limit": 1}
@@ -965,25 +964,29 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
             replies = await self._api("conversations.replies", _Schema.history, params=params)
         except SlackAPIError as e:
             log.debug("API error retrieving reply %r -> %r from %r: %r",
-                      thread_ts, reply_ts, channel_id, e.args[0])
+                      receipt.id, reply_ts, receipt.channel.source, e.args[0])
             raise MessageNotFound from None
         if replies["messages"]:
             msg = replies["messages"][-1]
             if msg["ts"] == reply_ts:
-                msg["channel"] = channel_id
+                msg["channel"] = receipt.channel.source
                 return await SlackMessage.from_event(self, msg, parent)
-        log.debug("Failed to find reply %r -> %r in %r", thread_ts, reply_ts, channel_id)
+        log.debug("Reply %r -> %r not found in %r", receipt.id, reply_ts, receipt.channel.source)
         raise MessageNotFound
 
     async def _post(self, channel, parent, msg):
         ids = []
         uploads = 0
+        name = None
+        data = {"channel": channel.source, "as_user": True}
         if msg.user:
-            name = msg.user.real_name or msg.user.username
-            image = msg.user.avatar
-        else:
-            name = self.config["fallback-name"]
-            image = self.config["fallback-image"]
+            data["as_user"] = False
+            data["username"] = name = msg.user.real_name or msg.user.username
+            if msg.user.avatar:
+                # Slack permanently caches user icon URLs; add monotonic hashes to keep them fresh.
+                data["icon_url"] = "{}#{}".format(msg.user.avatar, int(time.time()))
+            elif self.config["fallback-image"]:
+                data["icon_url"] = self.config["fallback-image"]
         for attach in msg.attachments:
             if isinstance(attach, immp.File):
                 # Upload each file to Slack.
@@ -995,8 +998,9 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
                     data.add_field("thread_ts", msg.reply_to.id)
                     if self.config["thread-broadcast"]:
                         data.add_field("broadcast", "true")
-                if msg.user:
-                    comment = immp.RichText([immp.Segment(name, bold=True, italic=True),
+                if name:
+                    comment = immp.RichText([immp.Segment(name, bold=True, italic=True,
+                                                          link=msg.user.link),
                                              immp.Segment(" uploaded this file", italic=True)])
                     data.add_field("initial_comment", SlackRichText.to_mrkdwn(self, comment))
                 img_resp = await attach.get_content(self.session)
@@ -1008,10 +1012,6 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
                         ids += [share["ts"] for share in shared[channel.source]]
         if len(ids) < uploads:
             log.warning("Missing some file shares: sent %d, got %d", uploads, len(ids))
-        data = {"channel": channel.source,
-                "as_user": msg.user is None,
-                "username": name,
-                "icon_url": image}
         rich = None
         if msg.text:
             rich = msg.text.clone()
