@@ -17,6 +17,8 @@ Config:
         at a later date, you'll receive a backlog of any messages not yet picked up.
     session (str):
         Optional path to store a session file, used to cache access hashes.
+    stickers (bool):
+        ``True`` to include stickers in messages as proprietary-format attachments (.tgs files).
 
 Bots can be created by talking to `@BotFather <https://t.me/BotFather>`_.  Use the ``/token``
 command to retrieve your API token.  You should also call ``/setprivacy`` to grant the bot
@@ -64,17 +66,18 @@ class _Schema:
                           immp.Optional("api-id"): immp.Nullable(int),
                           immp.Optional("api-hash"): immp.Nullable(str),
                           immp.Optional("client-updates", False): bool,
-                          immp.Optional("session"): immp.Nullable(str)})
+                          immp.Optional("session"): immp.Nullable(str),
+                          immp.Optional("stickers", True): bool})
 
     user = immp.Schema({"id": int,
                         immp.Optional("username"): immp.Nullable(str),
                         "first_name": str,
                         immp.Optional("last_name"): immp.Nullable(str)})
 
-    channel = immp.Schema({"id": int,
-                           "title": str,
-                           "type": "channel",
-                           immp.Optional("username"): immp.Nullable(str)})
+    chat = immp.Schema({"id": int,
+                        "type": str,
+                        immp.Optional("title"): immp.Nullable(str),
+                        immp.Optional("username"): immp.Nullable(str)})
 
     entity = immp.Schema({"type": str,
                           "offset": int,
@@ -82,18 +85,24 @@ class _Schema:
                           immp.Optional("url"): immp.Nullable(str),
                           immp.Optional("user"): immp.Nullable(user)})
 
-    _file = {"file_id": str, immp.Optional("file_name"): immp.Nullable(str)}
+    file = {"file_id": str,
+            immp.Optional("file_name"): immp.Nullable(str),
+            immp.Optional("file_path"): immp.Nullable(str)}
+
+    _sticker = {immp.Optional("emoji"): immp.Nullable(str), **file}
 
     _location = {"latitude": float, "longitude": float}
 
     message = immp.Schema({"message_id": int,
-                           "chat": {"id": int},
+                           "chat": chat,
                            "date": int,
                            immp.Optional("edit_date"): immp.Nullable(int),
                            immp.Optional("from"): immp.Nullable(user),
+                           immp.Optional("sender_chat"): immp.Nullable(chat),
+                           immp.Optional("author_signature"): immp.Nullable(str),
                            immp.Optional("forward_from"): immp.Nullable(user),
                            immp.Optional("forward_date"): immp.Nullable(int),
-                           immp.Optional("forward_from_chat"): immp.Nullable(channel),
+                           immp.Optional("forward_from_chat"): immp.Nullable(chat),
                            immp.Optional("forward_from_message_id"): immp.Nullable(int),
                            immp.Optional("forward_signature"): immp.Nullable(str),
                            immp.Optional("forward_sender_name"): immp.Nullable(str),
@@ -101,16 +110,14 @@ class _Schema:
                            immp.Optional("caption"): immp.Nullable(str),
                            immp.Optional("entities", list): [entity],
                            immp.Optional("caption_entities", list): [entity],
-                           immp.Optional("photo", list): [_file],
-                           immp.Optional("sticker"): immp.Nullable({immp.Optional("emoji"):
-                                                                        immp.Nullable(str),
-                                                                    "file_id": str}),
-                           immp.Optional("animation"): immp.Nullable(_file),
-                           immp.Optional("video"): immp.Nullable(_file),
-                           immp.Optional("video_note"): immp.Nullable(_file),
-                           immp.Optional("audio"): immp.Nullable(_file),
-                           immp.Optional("voice"): immp.Nullable(_file),
-                           immp.Optional("document"): immp.Nullable(_file),
+                           immp.Optional("photo", list): [file],
+                           immp.Optional("sticker"): immp.Nullable(_sticker),
+                           immp.Optional("animation"): immp.Nullable(file),
+                           immp.Optional("video"): immp.Nullable(file),
+                           immp.Optional("video_note"): immp.Nullable(file),
+                           immp.Optional("audio"): immp.Nullable(file),
+                           immp.Optional("voice"): immp.Nullable(file),
+                           immp.Optional("document"): immp.Nullable(file),
                            immp.Optional("location"): immp.Nullable(_location),
                            immp.Optional("venue"): immp.Nullable({"location": _location,
                                                                   "title": str,
@@ -121,7 +128,7 @@ class _Schema:
                            immp.Optional("new_chat_members", list): [user],
                            immp.Optional("left_chat_member"): immp.Nullable(user),
                            immp.Optional("new_chat_title"): immp.Nullable(str),
-                           immp.Optional("new_chat_photo", list): [_file],
+                           immp.Optional("new_chat_photo", list): [file],
                            immp.Optional("delete_chat_photo", False): bool,
                            immp.Optional("migrate_to_chat_id"): immp.Nullable(int)})
 
@@ -133,6 +140,7 @@ class _Schema:
                           immp.Optional(immp.Any("message", "edited_message",
                                                  "channel_post", "edited_channel_post")): message})
 
+    @staticmethod
     def api(result=None):
         success = {"ok": True}
         if result:
@@ -142,17 +150,6 @@ class _Schema:
                                      "description": str,
                                      "error_code": int}))
 
-    me = api(user)
-
-    file = api({"file_path": str})
-
-    send = api(message)
-
-    chat = api({"type": str,
-                immp.Optional("title"): immp.Nullable(str)})
-
-    updates = api([update])
-
 
 class TelegramAPIConnectError(immp.PlugError):
     """
@@ -166,15 +163,22 @@ class TelegramAPIRequestError(immp.PlugError):
     """
 
 
-class _HiddenSender(Exception):
+class _HiddenSender:
 
-    channel_id = -1001228946795
-    chat_id = 1228946795
+    # @HiddenSender, "a user": author of message forwards when opted to be linked back to them.
+    hidden_chat_id = 1228946795
+    hidden_channel_id = int("-100{}".format(hidden_chat_id))
+
+    # "Telegram": official service account, author of channel messages relayed to discussion chats.
+    service_user_id = 777000
+
+    # @GroupAnonymousBot, "Group": author of group chat messages on behalf of anonymous admins.
+    anonymous_user_id = 1087968824
 
     @classmethod
-    def check(cls, value):
-        if value in (cls.channel_id, cls.chat_id):
-            raise cls
+    def has(cls, value):
+        return value in (cls.hidden_channel_id, cls.hidden_chat_id,
+                         cls.service_user_id, cls.anonymous_user_id)
 
 
 class TelegramUser(immp.User):
@@ -212,7 +216,7 @@ class TelegramUser(immp.User):
                    raw=user)
 
     @classmethod
-    def from_bot_channel(cls, telegram, json):
+    def from_bot_channel(cls, telegram, json, author=None):
         """
         Convert a chat :class:`dict` (attached to a message) to a :class:`.User`.
 
@@ -221,17 +225,18 @@ class TelegramUser(immp.User):
                 Related plug instance that provides the user.
             json (dict):
                 Telegram API `Chat <https://core.telegram.org/bots/api#chat>`_ object.
+            author (str):
+                Optional post author, for channel messages.
 
         Returns:
             .TelegramUser:
                 Parsed user object.
         """
-        chat = _Schema.channel(json)
-        _HiddenSender.check(chat["id"])
+        chat = _Schema.chat(json)
         return cls(id_=chat["id"],
                    plug=telegram,
                    username=chat["username"],
-                   real_name=chat["title"],
+                   real_name=author or chat["title"],
                    raw=chat)
 
     @classmethod
@@ -263,14 +268,14 @@ class TelegramUser(immp.User):
                    raw=user)
 
     @classmethod
-    def from_proto_channel(cls, telegram, user, author=None):
+    def from_proto_channel(cls, telegram, chat, author=None):
         """
         Convert a :class:`telethon.tl.types.Channel` into a :class:`.User`.
 
         Args:
             telegram (.TelegramPlug):
                 Related plug instance that provides the user.
-            user (telethon.tl.types.Channel):
+            chat (telethon.tl.types.Channel):
                 Telegram channel retrieved from the MTProto API.
             author (str):
                 Optional post author, for channel messages.
@@ -279,12 +284,11 @@ class TelegramUser(immp.User):
             .TelegramUser:
                 Parsed user object.
         """
-        _HiddenSender.check(user.id)
-        return cls(id_=user.id,
+        return cls(id_="-100{}".format(chat.id),
                    plug=telegram,
-                   username=user.username,
-                   real_name=author or user.title,
-                   raw=user)
+                   username=chat.username,
+                   real_name=author or chat.title,
+                   raw=chat)
 
     @classmethod
     def from_entity(cls, telegram, entity):
@@ -585,8 +589,11 @@ class TelegramMessage(immp.Message):
         left = None
         title = None
         attachments = []
-        if message["from"]:
+        if message["from"] and not _HiddenSender.has(message["from"]["id"]):
             user = TelegramUser.from_bot_user(telegram, message["from"])
+        elif message["sender_chat"] and not _HiddenSender.has(message["sender_chat"]["id"]):
+            user = TelegramUser.from_bot_chat(telegram, message["sender_chat"],
+                                              message["author_signature"])
         if message["reply_to_message"]:
             reply_to = await cls.from_bot_message(telegram, message["reply_to_message"])
         # At most one of these fields will be set.
@@ -641,8 +648,10 @@ class TelegramMessage(immp.Message):
                 text = await TelegramRichText.from_bot_entities(telegram, message["caption"],
                                                                 message["caption_entities"])
         elif message["sticker"]:
-            attachments.append(await TelegramFile.from_id(telegram, message["sticker"]["file_id"],
-                                                          immp.File.Type.image))
+            if telegram.config["stickers"] or not message["sticker"]["emoji"]:
+                attachments.append(await TelegramFile.from_id(telegram,
+                                                              message["sticker"]["file_id"],
+                                                              immp.File.Type.image))
             # All real stickers should have an emoji, but webp images uploaded as photos are
             # incorrectly categorised as stickers in the API response.
             if not text and message["sticker"]["emoji"]:
@@ -687,19 +696,16 @@ class TelegramMessage(immp.Message):
             # Event is a message containing another message.  Forwarded messages have no ID, so we
             # use a Message instead of a SentMessage here, unless they come from a channel.
             forward_id = forward_channel = forward_user = None
-            if message["forward_from_chat"]:
-                forward_channel = immp.Channel(telegram, message["forward_from_chat"]["id"])
-                try:
-                    forward_user = TelegramUser.from_bot_channel(telegram,
-                                                                 message["forward_from_chat"])
-                except _HiddenSender:
-                    if message["forward_signature"]:
-                        forward_user = immp.User(real_name=message["forward_signature"])
-                if message["forward_from_message_id"]:
-                    forward_id = "{}:{}".format(message["forward_from_chat"]["id"],
-                                                message["forward_from_message_id"])
-            elif message["forward_from"]:
-                forward_user = TelegramUser.from_bot_user(telegram, message["forward_from"])
+            chat = message["forward_from_chat"]
+            sender = message["forward_from"]
+            if chat and message["forward_from_message_id"]:
+                forward_channel = immp.Channel(telegram, chat["id"])
+                forward_id = "{}:{}".format(chat["id"], message["forward_from_message_id"])
+            if sender and not _HiddenSender.has(sender["id"]):
+                forward_user = TelegramUser.from_bot_user(telegram, sender)
+            elif chat and not _HiddenSender.has(chat["id"]):
+                forward_user = TelegramUser.from_bot_channel(telegram, chat,
+                                                             message["forward_signature"])
             elif message["forward_sender_name"]:
                 forward_user = immp.User(real_name=message["forward_sender_name"])
             forward_common = dict(text=text,
@@ -778,11 +784,13 @@ class TelegramMessage(immp.Message):
             revision = None
         text = await TelegramRichText.from_proto_entities(telegram, message.message,
                                                           message.entities)
-        sender = await message.get_sender()
-        if isinstance(sender, tl.types.Channel):
-            user = TelegramUser.from_proto_channel(telegram, sender)
-        else:
+        user = None
+        if message.sender_id and not _HiddenSender.has(message.sender_id):
+            sender = await message.get_sender()
             user = TelegramUser.from_proto_user(telegram, sender)
+        elif message.chat_id and not _HiddenSender.has(message.chat_id):
+            chat = await message.get_chat()
+            user = TelegramUser.from_proto_channel(telegram, chat, message.post_author)
         action = False
         reply_to = None
         joined = []
@@ -803,25 +811,29 @@ class TelegramMessage(immp.Message):
         elif message.document:
             type_ = immp.File.Type.unknown
             name = None
+            sticker = False
             for attr in message.document.attributes:
                 if isinstance(attr, tl.types.DocumentAttributeSticker):
                     type_ = immp.File.Type.image
                     if attr.alt and not text:
                         text = "sent {} sticker".format(attr.alt)
                         action = True
+                        sticker = True
                 elif isinstance(attr, tl.types.DocumentAttributeAnimated):
                     type_ = immp.File.Type.image
                 elif isinstance(attr, tl.types.DocumentAttributeVideo):
                     type_ = immp.File.Type.video
                 if isinstance(attr, tl.types.DocumentAttributeFilename):
                     name = attr.file_name
-            try:
-                attach = await TelegramFile.from_id(telegram, pack_bot_file_id(message.document),
-                                                    type_, name)
-            except TelegramAPIRequestError as e:
-                log.warning("Unable to fetch attachment", exc_info=e)
-            else:
-                attachments.append(attach)
+            if telegram.config["stickers"] or not sticker:
+                try:
+                    attach = await TelegramFile.from_id(telegram,
+                                                        pack_bot_file_id(message.document),
+                                                        type_, name)
+                except TelegramAPIRequestError as e:
+                    log.warning("Unable to fetch attachment", exc_info=e)
+                else:
+                    attachments.append(attach)
         elif message.poll:
             action = True
             prefix = "closed the" if message.poll.poll.closed else "opened a"
@@ -879,20 +891,17 @@ class TelegramMessage(immp.Message):
             # Event is a message containing another message.  Forwarded messages have no ID, so we
             # use a Message instead of a SentMessage here, unless they come from a channel.
             forward_id = forward_channel = forward_user = None
-            if message.forward.channel_id and message.forward.channel_post:
+            if message.forward.chat_id and message.forward.channel_post:
                 forward_channel = immp.Channel(telegram, message.forward.chat_id)
                 forward_id = "{}:{}".format(message.forward.chat_id,
                                             message.forward.channel_post)
+            if message.forward.sender_id and not _HiddenSender.has(message.forward.sender_id):
+                sender = await message.forward.get_sender()
+                forward_user = TelegramUser.from_proto_user(telegram, sender)
+            elif message.forward.chat_id and not _HiddenSender.has(message.forward.chat_id):
                 chat = await message.forward.get_chat()
-                try:
-                    forward_user = TelegramUser.from_proto_channel(telegram, chat,
-                                                                   message.forward.post_author)
-                except _HiddenSender:
-                    if message.forward.post_author:
-                        forward_user = immp.User(real_name=message.forward.post_author)
-            elif message.forward.sender_id:
-                forward_user = TelegramUser.from_proto_user(telegram,
-                                                            await message.forward.get_sender())
+                forward_user = TelegramUser.from_proto_channel(telegram, chat,
+                                                               message.forward.post_author)
             elif message.forward.from_name:
                 forward_user = immp.User(real_name=message.forward.from_name)
             forward_common = dict(text=text,
@@ -989,7 +998,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         # highest we've seen, so that we can attempt to fetch past messages with this as a base.
         self._last_id = None
 
-    async def _api(self, endpoint, schema=_Schema.api(), quiet=False, **kwargs):
+    async def _api(self, endpoint, type_=None, quiet=False, **kwargs):
         url = "https://api.telegram.org/bot{}/{}".format(self.config["token"], endpoint)
         if not quiet:
             log.debug("Making API request to %r", endpoint)
@@ -997,6 +1006,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             async with self.session.post(url, **kwargs) as resp:
                 try:
                     json = await resp.json()
+                    schema = _Schema.api(type_)
                     data = schema(json)
                 except ClientResponseError as e:
                     raise TelegramAPIConnectError("Bad response with code: {}"
@@ -1012,7 +1022,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
     async def start(self):
         await super().start()
         self._closing = False
-        self._bot_user = await self._api("getMe", _Schema.me)
+        self._bot_user = await self._api("getMe", _Schema.user)
         if self.config["api-id"] and self.config["api-hash"]:
             if not TelegramClient:
                 raise immp.ConfigError("API ID/hash specified but Telethon is not installed")
@@ -1040,7 +1050,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
                 pass
             elif diff.new_messages:
                 self._last_id = diff.new_messages[-1].id
-            self._blacklist = {_HiddenSender.channel_id}
+            self._blacklist = {_HiddenSender.hidden_channel_id}
             self._blacklist_task = ensure_future(wait([self._blacklist_users(),
                                                        self._blacklist_chats()]))
 
@@ -1067,6 +1077,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             log.warning("Chat migrations require a config update before next run")
 
     async def user_from_id(self, id_):
+        id_ = int(id_)
         if not self._client:
             log.debug("Client auth required to look up users")
             return None
@@ -1076,7 +1087,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         elif id_ in self._users:
             return self._users[id_]
         try:
-            data = await self._client(tl.functions.users.GetFullUserRequest(int(id_)))
+            data = await self._client(tl.functions.users.GetFullUserRequest(id_))
         except ValueError:
             log.warning("Missing entity for user %d", id_)
             return None
@@ -1115,7 +1126,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         count = 0
         for user in self._client.session.get_user_entities():
             try:
-                await self._api("getChat", quiet=True, params={"chat_id": user[0]})
+                await self._api("getChat", _Schema.chat, quiet=True, params={"chat_id": user[0]})
             except TelegramAPIRequestError:
                 count += 1
                 self._blacklist.add(user[0])
@@ -1168,7 +1179,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         if not isinstance(user, TelegramUser):
             return None
         try:
-            await self._api("getChat", params={"chat_id": user.id})
+            await self._api("getChat", _Schema.chat, params={"chat_id": user.id})
         except TelegramAPIRequestError as e:
             log.warning("Failed to retrieve user %s channel", user.id, exc_info=e)
             # Can't create private channels, users must initiate conversations with bots.
@@ -1219,7 +1230,8 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
                     data = await self._client(tl.functions.channels.GetParticipantsRequest(
                         chat, tl.types.ChannelParticipantsRecent(), len(users), 1000, 0))
                     if data.users:
-                        users += [TelegramUser.from_proto_user(self, user) for user in data.users]
+                        users += [TelegramUser.from_proto_user(self, user) for user in data.users
+                                  if not _HiddenSender.has(user.id)]
                     else:
                         break
             except ValueError:
@@ -1306,66 +1318,90 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             data.add_field(field, img_resp.content, filename=attach.title or field)
         return data
 
-    async def _upload_attachment(self, chat, msg, attach):
+    async def _upload_attachment(self, chat, msg, attach, reply_to=None, caption=None):
         # Upload a file to Telegram in its own message.
         # Prefer a source URL if available, else fall back to re-uploading the file.
         base = {"chat_id": str(chat)}
+        if reply_to:
+            base.update({"reply_to_message_id": reply_to,
+                         "allow_sending_without_reply": "true"})
         if msg.user:
-            if attach.type == immp.File.Type.image:
-                what = "an image"
-            elif attach.type == immp.File.Type.video:
-                what = "a video"
-            else:
-                what = "a file"
-            rich = immp.Message(text=immp.RichText([immp.Segment("sent {}".format(what))]),
-                                user=msg.user, action=True).render()
-            text = "".join(TelegramSegment.to_html(self, segment) for segment in rich)
+            if not caption:
+                if attach.type == immp.File.Type.image:
+                    what = "an image"
+                elif attach.type == immp.File.Type.video:
+                    what = "a video"
+                else:
+                    what = "a file"
+                caption = immp.Message(text=immp.RichText([immp.Segment("sent {}".format(what))]),
+                                       user=msg.user, action=True).render()
+            text = "".join(TelegramSegment.to_html(self, segment) for segment in caption)
             base["caption"] = text
             base["parse_mode"] = "HTML"
         if attach.type == immp.File.Type.image:
             data = await self._form_data(base, "photo", attach)
             try:
-                return await self._api("sendPhoto", _Schema.send, data=data)
+                return await self._api("sendPhoto", _Schema.message, data=data)
             except (TelegramAPIConnectError, TelegramAPIRequestError):
                 log.debug("Failed to upload image, falling back to document upload")
         elif attach.type == immp.File.Type.video:
             data = await self._form_data(base, "video", attach)
             try:
-                return await self._api("sendVideo", _Schema.send, data=data)
+                return await self._api("sendVideo", _Schema.message, data=data)
             except (TelegramAPIConnectError, TelegramAPIRequestError):
                 log.debug("Failed to upload video, falling back to document upload")
         data = await self._form_data(base, "document", attach)
         try:
-            return await self._api("sendDocument", _Schema.send, data=data)
+            return await self._api("sendDocument", _Schema.message, data=data)
         except TelegramAPIConnectError as e:
             log.warning("Failed to upload file", exc_info=e)
             return None
 
     def _requests(self, chat, msg):
-        requests = []
-        if msg.text or msg.reply_to:
-            quote = False
-            reply_to = ""
-            if isinstance(msg.reply_to, immp.Receipt):
-                # Reply natively to the given parent message.
-                reply_to = int(msg.reply_to.id.split(":")[1])
-            elif isinstance(msg.reply_to, immp.Message):
-                quote = True
+        reply_to = ""
+        quote = False
+        rich = None
+        if isinstance(msg.reply_to, immp.Receipt):
+            reply_to = int(msg.reply_to.id.split(":")[1])
+        elif isinstance(msg.reply_to, immp.Message):
+            quote = True
+        if msg.text or quote:
             rich = msg.render(edit=msg.edited, quote_reply=quote)
+        # If there's exactly one file attachment that we can attach a caption to, use that for the
+        # message text, otherwise send the text first in its own message.
+        parts = []
+        captionable = []
+        for attach in msg.attachments:
+            if isinstance(attach, immp.File):
+                parts.append(attach)
+                captionable.append(attach)
+            elif isinstance(attach, immp.Location):
+                parts.append(attach)
+        requests = []
+        # Send the primary attachment, or the leading text-only message, with any reply metadata.
+        primary = None
+        if len(captionable) == 1:
+            primary = captionable[0]
+            requests.append(self._upload_attachment(chat, msg, primary, reply_to, rich))
+        elif rich:
             text = "".join(TelegramSegment.to_html(self, segment) for segment in rich)
+            # Prevent linked user names generating link previews.
             no_link_preview = "true" if msg.user and msg.user.link else "false"
-            requests.append(self._api("sendMessage", _Schema.send,
+            requests.append(self._api("sendMessage", _Schema.message,
                                       params={"chat_id": chat,
                                               "text": text,
                                               "parse_mode": "HTML",
-                                              # Prevent linked user names generating previews.
                                               "disable_web_page_preview": no_link_preview,
-                                              "reply_to_message_id": reply_to}))
+                                              "reply_to_message_id": reply_to,
+                                              "allow_sending_without_reply": "true"}))
+        # Send any remaining attachments as auxilary messages.
         for attach in msg.attachments:
-            if isinstance(attach, immp.File):
+            if attach is primary:
+                continue
+            elif isinstance(attach, immp.File):
                 requests.append(self._upload_attachment(chat, msg, attach))
             elif isinstance(attach, immp.Location):
-                requests.append(self._api("sendLocation", _Schema.send,
+                requests.append(self._api("sendLocation", _Schema.message,
                                           params={"chat_id": chat,
                                                   "latitude": str(attach.latitude),
                                                   "longitude": str(attach.longitude)}))
@@ -1373,7 +1409,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
                     caption = immp.Message(user=msg.user, text="sent a location", action=True)
                     text = "".join(TelegramSegment.to_html(self, segment)
                                    for segment in caption.render())
-                    requests.append(self._api("sendMessage", _Schema.send,
+                    requests.append(self._api("sendMessage", _Schema.message,
                                               params={"chat_id": chat,
                                                       "text": text,
                                                       "parse_mode": "HTML"}))
@@ -1390,7 +1426,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             if isinstance(attach, immp.Receipt):
                 # Forward the messages natively using the given chat/ID.
                 forward_chat, forward_id = map(int, attach.id.split(":"))
-                requests.append(self._api("forwardMessage", _Schema.send,
+                requests.append(self._api("forwardMessage", _Schema.message,
                                           params={"chat_id": chat,
                                                   "from_chat_id": forward_chat,
                                                   "message_id": forward_id}))
@@ -1436,7 +1472,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         while not self._closing:
             params = {"offset": self._offset,
                       "timeout": 240}
-            fetch = ensure_future(self._api("getUpdates", _Schema.updates, params=params))
+            fetch = ensure_future(self._api("getUpdates", [_Schema.update], params=params))
             try:
                 result = await fetch
             except CancelledError:
